feature_template = """Feature: Example Feature
  Background: Configure Your Test
    Given some preconditions
    
  Scenario: Test Your App
    Given scenario specific preconditions
    Then verify results
    
"""


from mitmproxy import http
from typing import List


class ProxyDelegate:

    def __init__(self, recursive: bool = True) -> None:
        """
        Delegate all jobs you need to be done by the proxy to instance of this class
        :param recursive: When True all previous requests will be feed into the delegate when it's added
        """
        super().__init__()
        self.recursive = recursive

    def request(self, flow: http.HTTPFlow): pass

    def response(self, flow: http.HTTPFlow): pass

    def shutdown(self): pass


class ProxyPlugin:
    """
    Base plugin implementation  for mitmproxy
    """

    def __init__(self):
        self._requests: List[http.HTTPFlow] = []
        self._delegates: List[ProxyDelegate] = []
        self.proxy = None

    def add_delegate(self, delegate: ProxyDelegate):
        self._delegates.append(delegate)
        if delegate.recursive:
            for flow in self._requests:
                delegate.request(flow)
                if flow.response:
                    delegate.response(flow)

    def load(self, loader):
        pass

    def configure(self, updated):
        pass

    def request(self, flow: http.HTTPFlow):
        self._requests.append(flow)
        for delegate in self._delegates:
            delegate.request(flow)

    def response(self, flow: http.HTTPFlow):
        for delegate in self._delegates:
            delegate.response(flow)

    def reset(self):
        self._requests = []
        self._delegates = []

    def shutdown(self):
        for delegate in self._delegates:
            delegate.shutdown()
        if self.proxy:
            self.proxy.shutdown()



from behave import given, then

from rikki.behave.context import Context
from rikki.behave.steps.utils import http_data
from rikki.behave.steps.utils import verify_number_of_request
from rikki.mitmproxy.delegate_replace import ReplacementConfig, InterceptAndReplace


@given("Intercept and replace http data")
def step_replace_http_data(context: Context):
    http_step_data = http_data(context)

    if http_step_data:
        config = ReplacementConfig(
            filter_request=http_step_data.filter_request,
            filter_response=http_step_data.filter_response,
            replacement_request=http_step_data.replacement_request,
            replacement_response=http_step_data.replacement_response,
            inclusive=False
        )
        plugin = InterceptAndReplace(config=config)
        context.proxy.add_delegate(plugin)


@then('verify {number:d} requests. With tag: "{tag}"')
def step_verify_number_of_requests(context: Context, number, tag):
    verify_number_of_request(
        context=context,
        number=number,
        tag=tag,
        second_attempt=True
    )



from typing import Dict, Optional


class Request:
    def __init__(
            self,
            host: Optional[str] = None,
            params: Optional[Dict[str, object]] = None,
            headers: Optional[Dict[str, str]] = None,
            content: Optional[str] = None
    ) -> None:
        super().__init__()
        self.host: Optional[str] = host
        self.params: Optional[Dict[str, object]] = params
        self.headers: Optional[Dict[str, str]] = headers
        self.content: Optional[str] = content

    def empty(self) -> bool:
        if self.host or self.content or self.params or self.headers:
            return False
        else:
            return True


class Response:
    def __init__(
            self,
            headers: Optional[Dict[str, str]] = None,
            content: Optional[str] = None,
            code: Optional[int] = None
    ) -> None:
        super().__init__()
        self.headers: Optional[Dict[str, str]] = headers
        self.content: Optional[str] = content
        self.code: Optional[int] = code

    def empty(self) -> bool:
        if self.code or self.headers or self.content:
            return False
        else:
            return True



import os
from pathlib import Path
from rikki.template.environment import environment_template
from rikki.template.feature import feature_template
from rikki.template.run_file import run_file_template


def behave_init():
    features_folder_name = "features"
    resource_folder_name = "res"

    if not os.path.exists(Path(features_folder_name)):
        os.makedirs(Path(features_folder_name))

    if not os.path.exists(Path(resource_folder_name)):
        os.makedirs(Path(resource_folder_name))

    steps_folder_path = Path("{0}/steps".format(features_folder_name))

    if not os.path.exists(steps_folder_path):
        os.makedirs(steps_folder_path)

    with open(Path("features/environment.py"), "w") as environment:
        environment.write(environment_template)

    with open(Path("features/example.feature"), "w") as example_feature:
        example_feature.write(feature_template)

    with open(Path("run.py"), "w") as run_file:
        run_file.write(run_file_template)



from enum import Enum
from typing import Optional

from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support import expected_conditions as conditions
from selenium.webdriver.support.wait import WebDriverWait


class Direction(Enum):
    UP = 1
    DOWN = 2
    LEFT = 3
    RIGHT = 4


class SwipeSpeed(Enum):
    SLOW = 200
    MEDIUM = 400
    FAST = 800
    ROCKET_MAN = 1600


class AppiumUtils:

    def __init__(self, driver: WebDriver, wait_time: int = 1) -> None:
        super().__init__()
        self.wait_time = wait_time
        self._driver: WebDriver = driver

    def tap(self, by: By, locator: str, wait_time: Optional[int] = None):
        wait = self._configure_wait(wait_time)
        wait.until(conditions.presence_of_element_located((by, locator))).click()

    def enter(self, by: By, locator: str, text: str, wait_time: Optional[int] = None):
        wait = self._configure_wait(wait_time)
        wait.until(conditions.presence_of_element_located((by, locator))).send_keys(text)

    def wait(self, by: By, locator: str, wait_time: Optional[int] = None):
        wait = self._configure_wait(wait_time)
        wait.until(conditions.presence_of_element_located((by, locator)))

    def relaunch_app(self):
        assert False, "Relaunch app is not implemented"

    def swipe(
            self,
            element: Optional[object] = None,
            direction: Direction = Direction.DOWN,
            speed: SwipeSpeed = SwipeSpeed.MEDIUM
    ):
        assert False, "Swipe is not supported:"

    def _configure_wait(self, wait_time: Optional[int] = None):
        delay = self.wait_time
        if wait_time:
            delay = wait_time
        return WebDriverWait(self._driver, delay)

    @staticmethod
    def support(driver: WebDriver) -> bool:
        """
        :param driver:
        :return: True if this instance of the utils support the context
        """
        return True




environment_template = """
from allure_behave.hooks import allure_report
from appium import webdriver
from appium.webdriver.appium_service import AppiumService
from behave.model import Scenario
from rikki.behave.context import Context


def before_all(ctx: Context):
    setattr(ctx, "appium", AppiumService())
    ctx.appium.start()
    setattr(ctx, "proxy", ctx.config.proxy)


def after_all(ctx: Context):
    ctx.proxy.shutdown()
    ctx.appium.stop()


def before_scenario(ctx: Context, scenario: Scenario):
    # start browser
    ctx.proxy.reset()
    #     Android capabilities
    #     desired_capabilities = {
    #         "platformName": "Android",
    #         "deviceName": None,
    #         "automationName": "uiautomator2",
    #         "appPackage": None,
    #         "appActivity": None,
    #         "noReset": "noReset" in scenario.tags,
    #         "app": None
    #     }
    #     iOS Capabilities
    #     desired_capabilities = {
    #         "platformName": "iOS",
    #         "deviceName": None,
    #         "platformVersion": None
    #         "automationName": "XCUITest",
    #         "noReset": "noReset" in scenario.tags,
    #         "app": None
    #     }
    #     Decide which utils to use
    #     setattr(ctx, "appium_utils", Choose From one of AppiumUtils subclasses)
    ctx.browser = webdriver.Remote('http://localhost:4723/wd/hub', desired_capabilities)

    ctx.browser.implicitly_wait(10)


def after_scenario(ctx: Context, scenario: Scenario):
    ctx.browser.quit()
    ctx.proxy.reset()


allure_report("./reports")

"""



from typing import Optional

from selenium.webdriver import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webdriver import WebDriver

from rikki.appium.common import AppiumUtils, Direction, SwipeSpeed


class AndroidUiAutomator2Utils(AppiumUtils):

    def relaunch_app(self):
        self._driver.launch_app()

    def swipe(
            self,
            element: Optional[object] = None,
            direction: Direction = Direction.DOWN,
            speed: SwipeSpeed = SwipeSpeed.MEDIUM
    ):
        if direction == Direction.UP:
            self.__do_swipe(0, speed.value, element)
        if direction == Direction.DOWN:
            self.__do_swipe(0, -speed.value, element)
        if direction == Direction.LEFT:
            self.__do_swipe(-speed.value, 0, element)
        if direction == Direction.RIGHT:
            self.__do_swipe(speed.value, 0, element)

    def __do_swipe(self, x_offset: int, y_offset: int, element=None):
        if not element:
            element = self._driver.find_elements_by_xpath('//*')[0]

        if element:
            actions = ActionChains(self._driver)
            actions.click_and_hold(element)
            actions.move_by_offset(x_offset, y_offset)
            actions.perform()

    def enable_proxy(self):
        """
        Requires Appium Service to be started with `--relaxed-security`
        Will try to setup proxy on the device and relaunch app.
        """
        self._driver.execute_script("mobile: shell", "am start -a android.settings.WIRELESS_SETTINGS")
        self.tap(By.XPATH, "//*[contains(@text,'AndroidWifi')]")
        self.tap(By.ID, "Settings")
        self.tap(By.ID, "Modify")
        self.tap(By.ID, "com.android.settings:id/proxy_settings")
        self.tap(By.XPATH, "//*[contains(@text,'Manual')]")
        self.enter(By.ID, "com.android.settings:id/proxy_hostname", "10.0.2.2")
        self.enter(By.ID, "com.android.settings:id/proxy_port", "8080")
        self.tap(By.ID, "android:id/button1")
        self.relaunch_app()

    @staticmethod
    def support(driver: WebDriver) -> bool:
        platform = driver.desired_capabilities['desired']['platformName']
        automation = driver.desired_capabilities['desired']['automationName']

        return platform == "Android" and automation == "UiAutomator2"



from unittest import TestCase

from mitmproxy import http

from rikki.mitmproxy.delegate_replace import InterceptAndReplace, ReplacementConfig
from rikki.mitmproxy.model import Request, Response
from rikki.mitmproxy.util import contains_headers, contains_params
from test.rikki.mitmproxy.utils import build_flow


class TestReplacementDelegate(TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.__filter_request = Request(
            host="example.com",
            content="example content",
            headers={"header1": "header1_value"},
            params={"param1": "param1_value"},
        )

        self.__replacement_request = Request(
            host="new_example.com",
            content="new content",
            headers={"new_header1": "new_header1_value"},
            params={"new_param1": "new_param1_value"},
        )

        self.__filter_response = Response(
            code=200,
            content="response_content",
            headers={"response_header_1": "response_header_1_value"}
        )

        self.__replacement_response = Response(
            code=300,
            content="new_response_content",
            headers={"new_response_header_1": "new_response_header_1_value"}
        )

    def test_request_replacement_by_request(self):
        flow: http.HTTPFlow = build_flow(
            request=self.__filter_request
        )

        config = ReplacementConfig(
            filter_request=self.__filter_request,
            replacement_request=self.__replacement_request
        )

        instance = InterceptAndReplace(config)

        instance.request(flow)

        assert flow.request.host == self.__replacement_request.host, "Host hasn't been replaced"
        assert flow.request.content == self.__replacement_request.content.encode("utf8"), "Content hasn't been replaced"
        assert contains_params(flow.request.query, self.__replacement_request.params), "Params hasn't been replaced"
        assert contains_headers(flow.request.headers,
                                self.__replacement_request.headers), "Headers hasn't been replaced"

    def test_response_replacement_by_request(self):
        flow: http.HTTPFlow = build_flow(
            request=self.__filter_request,
            response=self.__filter_response
        )

        config = ReplacementConfig(
            filter_request=self.__filter_request,
            replacement_response=self.__replacement_response
        )

        instance = InterceptAndReplace(config)

        instance.response(flow)

        assert flow.response.status_code == self.__replacement_response.code, "Response code hasn't been changed"
        assert contains_headers(flow.response.headers,
                                self.__replacement_response.headers), "Response headers hasn't been replaced"
        assert flow.response.content == self.__replacement_response.content.encode("utf8")

    def test_response_replacement_by_request_and_response(self):
        flow: http.HTTPFlow = build_flow(
            request=self.__filter_request,
            response=self.__filter_response
        )

        config = ReplacementConfig(
            filter_request=self.__filter_request,
            filter_response=self.__filter_response,
            replacement_response=self.__replacement_response
        )

        instance = InterceptAndReplace(config)
        instance.response(flow)

        assert flow.response.status_code == self.__replacement_response.code, "Response code hasn't been changed"
        assert contains_headers(flow.response.headers,
                                self.__replacement_response.headers), "Response headers hasn't been replaced"
        assert flow.response.content == self.__replacement_response.content.encode("utf8")

    def test_inclusive_replacement(self):
        flow: http.HTTPFlow = build_flow(
            request=self.__filter_request,
            response=self.__filter_response
        )

        replacement_request = Request(
            headers=self.__replacement_request.headers,
            params=self.__replacement_request.params
        )

        replacement_response = Response(
            headers=self.__replacement_response.headers
        )

        config = ReplacementConfig(
            filter_request=self.__filter_request,
            filter_response=self.__filter_response,
            replacement_request=replacement_request,
            replacement_response=replacement_response,
            inclusive=True
        )

        instance = InterceptAndReplace(config)
        instance.request(flow)
        instance.response(flow)

        assert len(flow.request.headers.items()) == len(config.replacement_request.headers.items())
        assert len(flow.request.query.items()) == len(config.replacement_request.params.items())
        assert len(flow.response.headers.items()) == len(config.replacement_response.headers.items())



from typing import Optional, List

from mitmproxy import http

from rikki.mitmproxy.model import Request, Response
from rikki.mitmproxy.plugin import ProxyDelegate
from rikki.mitmproxy.util import filter


class AccumulateFilterConfig:

    def __init__(
            self,
            request: Optional[Request] = None,
            response: Optional[Response] = None
    ) -> None:
        super().__init__()
        self.request = request
        self.response = response


class Accumulate(ProxyDelegate):

    def __init__(self, filter_config: AccumulateFilterConfig):
        """
        This plugin will accumulate all flows filtered with the config. Be aware that flow will appear in results
        with delay in case when you specify response in filter config
        :param filter_config: request and response data, that will be used to filter requests
        """
        super().__init__()
        self.__config = filter_config
        self.accumulated_items: List[http.HTTPFlow] = []

    def request(self, flow: http.HTTPFlow):
        if self.__config.request and not self.__config.response:
            self.accumulated_items.extend(filter([flow], request=self.__config.request))

    def response(self, flow: http.HTTPFlow):
        if self.__config.response:
            self.accumulated_items.extend(
                filter(
                    [flow],
                    request=self.__config.request,
                    response=self.__config.response
                )
            )



import os
import re
import time

from json import JSONDecoder
from pathlib import Path
from typing import Optional

from rikki.behave.context import Context
from rikki.mitmproxy.delegate_accumulate import AccumulateFilterConfig, Accumulate
from rikki.mitmproxy.model import Request, Response


def __take_not_empty_request(request: Request) -> Optional[Request]:
    if not request.empty():
        return request
    else:
        return None


def __take_not_empty_response(response: Response) -> Optional[Response]:
    if not response.empty():
        return response
    else:
        return None


def variable_for_key(key: str) -> str:
    if key.startswith("os."):
        var_name = key.replace("os.", "").upper()
        if var_name in os.environ:
            return os.environ[var_name]
        else:
            assert False, "Can't find environment variable {0}, did you forget to add it?".format(var_name)
    elif key.startswith("file."):
        file_name = key.replace("file.", "")
        path = Path("res/{0}".format(file_name))
        if os.path.exists(path):
            with open(path, "r") as file:
                return file.read()
        else:
            assert False, "Can't find file {0}, in {1}?".format(file_name, path.resolve())


def resolve_variables(candidate):
    result = candidate
    if isinstance(result, str):
        vars = re.findall(r'%{[^}%]+}%', result)
        for var in vars:
            result = result.replace(var, variable_for_key(var.replace('%{', '').replace('}%', '')))

    return result


class HttpStepData:

    def __init__(
            self,
            filter_request: Optional[Request] = None,
            filter_response: Optional[Response] = None,
            replacement_request: Optional[Request] = None,
            replacement_response: Optional[Response] = None
    ) -> None:
        self.filter_request: Optional[Request] = filter_request
        self.filter_response: Optional[Response] = filter_response
        self.replacement_request: Optional[Request] = replacement_request
        self.replacement_response: Optional[Response] = replacement_response

    def empty(self) -> bool:
        if self.filter_request or self.filter_response or self.replacement_request or self.replacement_response:
            return False
        else:
            return True


def http_data(context: Context) -> Optional[HttpStepData]:
    if context.table:
        json_decoder = JSONDecoder()

        filter_request = Request()
        filter_response = Response()
        replacement_request = Request()
        replacement_response = Response()

        for row in context.table:
            if row['option'] == 'host':
                filter_request.host = resolve_variables(row['value'])
            if row['option'] == 'headers':
                filter_request.headers = json_decoder.decode(resolve_variables(row['value']))
            if row['option'] == 'params':
                filter_request.params = json_decoder.decode(resolve_variables(row['value']))
            if row['option'] == 'content':
                filter_request.content = resolve_variables(row['value'])
            if row['option'] == 'response_code':
                filter_response.response_code = resolve_variables(row['value'])
            if row['option'] == 'response_headers':
                filter_response.headers = json_decoder.decode(resolve_variables(row['value']))
            if row['option'] == 'response_content':
                filter_response.response_content = resolve_variables(row['value'])
            if row['option'] == 'replace_host':
                replacement_request.host = resolve_variables(row['value'])
            if row['option'] == 'replace_headers':
                replacement_request.headers = json_decoder.decode(resolve_variables(row['value']))
            if row['option'] == 'replace_params':
                replacement_request.params = json_decoder.decode(resolve_variables(row['value']))
            if row['option'] == 'replace_content':
                replacement_request.content = resolve_variables(row['value'])
            if row['option'] == 'replace_response_code':
                replacement_response.response_code = resolve_variables(row['value'])
            if row['option'] == 'replace_response_content':
                replacement_response.response_content = resolve_variables(row['value'])
            if row['option'] == 'replace_response_headers':
                replacement_response.headers = json_decoder.decode(resolve_variables(row['value']))

        result = HttpStepData(
            filter_request=__take_not_empty_request(filter_request),
            filter_response=__take_not_empty_response(filter_response),
            replacement_request=__take_not_empty_request(replacement_request),
            replacement_response=__take_not_empty_response(replacement_response)
        )

        if not result.empty():
            return result
    return None


def verify_number_of_request(context: Context, number: int, tag: str, second_attempt: bool = False, delay: int = 5):
    """

    :param context: behave context to work with
    :param number: number of requests to verify
    :param tag: string to use with exception output to identify particular request
    :param second_attempt: if True will make delay and second attempt after specified delay
    :param delay: seconds to wait before next attempt
    """
    http_step_data = http_data(context)

    if http_step_data:
        config = AccumulateFilterConfig(
            request=http_step_data.filter_request,
            response=http_step_data.filter_response
        )
        plugin = Accumulate(filter_config=config)
        context.proxy.add_delegate(plugin)

        if second_attempt:
            assert number == len(plugin.accumulated_items), \
                "Expected number of requests {0} is {1}, but actual is {2}".format(
                    tag,
                    number,
                    len(plugin.accumulated_items)
                )
        else:
            time.sleep(delay)
            if number != len(plugin.accumulated_items):
                verify_number_of_request(context, number, tag, True)
